import numpy as np
import cv2
import math
from collections import OrderedDict


# In[deskew]:
def deskew(img):
    thresh = img
    edges = cv2.Canny(thresh, 50, 200, apertureSize=3)

    lines = cv2.HoughLines(edges, 1, np.pi / 1000, 55)
    try:
        d1 = OrderedDict()
        for i in range(len(lines)):
            for rho, theta in lines[i]:
                deg = np.rad2deg(theta)
                #                print(deg)
                if deg in d1:
                    d1[deg] += 1
                else:
                    d1[deg] = 1

        t1 = OrderedDict(sorted(d1.items(), key=lambda x: x[1], reverse=False))
        non_zero_pixels = cv2.findNonZero(thresh)
        center, wh, theta = cv2.minAreaRect(non_zero_pixels)
        angle = list(t1.keys())[0]
        if angle > 160:
            angle = 180 - angle
        if angle < 160 and angle > 20:
            angle = 12
        root_mat = cv2.getRotationMatrix2D(center, angle, 1)
        rows, cols = img.shape
        rotated = cv2.warpAffine(img, root_mat, (cols, rows), flags=cv2.INTER_CUBIC)

    except:
        rotated = img
        pass
    return rotated


def unshear(img):
    gray = img
    thresh = img.copy()
    # print(thresh)
    # plt.imshow(thresh)
    # plt.show()
    trans = thresh.transpose()

    arr = []
    for i in range(thresh.shape[1]):
        arr.insert(0, trans[i].sum())

    arr = []
    for i in range(thresh.shape[0]):
        arr.insert(0, thresh[i].sum())

    y = thresh.shape[0] - 1 - np.nonzero(arr)[0][0]
    y_top = thresh.shape[0] - 1 - np.nonzero(arr)[0][-1]

    trans1 = thresh.transpose()
    sum1 = []
    for i in range(trans1.shape[0]):
        sum1.insert(i, trans1[i].sum())

    height = y - y_top
    max_value = 255 * height
    prev_num = len([i for i in sum1 if i >= (0.6 * max_value)])
    final_ang = 0

    # # print(arr)
    # # print(x,y)
    for ang in range(-25, 25, 3):
        thresh = gray.copy()
        # print(thresh[0].shape)
        # print(ang)
        if ang > 0:
            # print(ang)
            for i in range(y):
                temp = thresh[i]
                move = int((y - i) * (math.tan(math.radians(ang))))
                if move >= temp.size:
                    move = temp.size
                thresh[i][:temp.size - move] = temp[move:]
                thresh[i][temp.size - move:] = [0 for m in range(move)]
        else:
            for i in range(y):
                temp = thresh[i]
                move = int((y - i) * (math.tan(math.radians(-ang))))
                if move >= temp.size:
                    move = temp.size
                thresh[i][move:] = temp[:temp.size - move]
                thresh[i][:move] = [0 for m in range(move)]

        trans1 = thresh.transpose()
        sum1 = []
        for i in range(trans1.shape[0]):
            sum1.insert(i, trans1[i].sum())
        num = len([i for i in sum1 if i >= (0.60 * max_value)])
        if (num >= prev_num):
            prev_num = num
            final_ang = ang

    thresh = gray.copy()
    if final_ang > 0:
        for i in range(y):
            temp = thresh[i]
            move = int((y - i) * (math.tan(math.radians(final_ang))))
            if move >= temp.size:
                move = temp.size
            thresh[i][:temp.size - move] = temp[move:]
            thresh[i][temp.size - move:] = [0 for m in range(move)]
    else:
        for i in range(y):
            temp = thresh[i]
            move = int((y - i) * (math.tan(math.radians(-final_ang))))
            if move >= temp.size:
                move = temp.size
            thresh[i][move:] = temp[:temp.size - move]
            thresh[i][:move] = [0 for m in range(move)]

    return thresh